require 'readline'
require 'yaml'
require 'zlib'

class TheGame::Game
  attr_accessor :name, :level, :items, :created_at, :saved_at

  I = TheGame::Item

  def initialize
    TheGame.initialize!
    @created_at = Time.now
    @saved_at   = Time.now
    @name       = "anonymous"
    @level      = 1
    @items      = [I.new("Gold"), I.new("Sword"), I.new("Orb of POWAH")]
  end
  
  def yaml_dump
    YAML.dump({
      'created_at' => @created_at,
      'saved_at'   => @saved_at,
      'name'       => @name,
      'level'      => @level
    })
  end

  def load(savefile = TheGame::SAVEFILE_PATH)
    yaml = YAML.load(read_savefile(savefile))
    @created_at = yaml['created_at']
    @saved_at   = yaml['saved_at']
    @name       = yaml['name']
    @level      = yaml['level']
  end
  
  def save(savefile = TheGame::SAVEFILE_PATH)
    @saved_at = Time.now
    Zlib::GzipWriter.open(savefile, Zlib::BEST_COMPRESSION) { |gz| gz << yaml_dump }
  end

  def read_savefile(savefile = TheGame::SAVEFILE_PATH)
    Zlib::GzipReader.open(savefile) { |gz| gz.read }
  end

  def set(key, value)
		case key.downcase.to_sym
		when :name then @name = value
		when :level then @level = value
		else raise ArgumentError, "Cannot set #{key}!" end
  end
  
  # Names are green,
  # locations are magenta,
  # levels are blue.
  def prompt
    "\e[1;32m#{@name} \e[35m/ \e[34m#{@level} $\e[0m "
  end

  def start!
    puts "~ the_game.rb v#{TheGame::Version::STRING} ~\nuse `help` for available commands"

    trap(:INT) { puts; exit }
    
    loop do
      input = ::Readline.readline(prompt, true)
      output = begin
        TheGame::Command.send(self, input)
      rescue => e
        e.pp
      end
      puts output unless input.blank? or output.blank?
    end
  end
end
